
import glob
import os
import re
import shutil
import sys
from gettext import gettext as _
from xml.dom.minidom import parse
from xml.parsers.expat import ExpatError

import gtk
import gc

from mayanna_globals import mayanna_is_panel_applet
from mayanna_base import Item, ItemSource, Topic, mayanna_get_topic_for_uri
from mayanna_file import FileItem
from mayanna_recent import RecentlyUsedOfMimeType, RecentAggregate
from mayanna_util import *
from mayanna_trash import trash_monitor
from mayanna_computer import DriveItem
from mayanna_tomboy import tomboy_source
from mayanna_prefs import Preferences
#
# Sidebar ItemSources
#

class DirectorySource(ItemSource):
    def __init__(self, path, name = None, icon = "stock_folder"):
        self.path = os.path.expanduser(path)

        if not name:
            name = os.path.basename(self.path).decode(sys.getfilesystemencoding(), "replace")
        ItemSource.__init__(self, name=name, icon=icon)

        self.monitor = FileMonitor(self.path)
        self.monitor.connect("event", lambda m, uri, ev: self.emit("reload"))
        self.monitor.open()
        self.paths=[]
        self.temp_items = None

    def get_enabled(self):
        return os.path.exists(self.path)

    def _add_name(self, paths, dirname, names):
        for name in names:
            paths.append(os.path.join(dirname, name))

    def get_items_uncached(self):
        self.paths = []
       # delself.temp_items
        self.temp_items=[]
        os.path.walk(self.path, self._add_name, self.paths)
        for path in self.paths:
            self.temp_items.append( FileItem(path))
           # delpath
       # delself.paths
        gc.collect()
        return self.temp_items

class EpiphanyDownloadsSource(ItemSource):
    def __init__(self, name = "Epiphany Downloads"):
        ItemSource.__init__(self, name=name)

        self.ephy_directory_source = None
        self.ephy_gconf_bridge = GConfBridge("/apps/epiphany/directories/")
        self.ephy_gconf_bridge.connect("changed::downloads_folder",
                                       lambda gb: self._ephy_downloads_folder_changed())
        self._ephy_downloads_folder_changed()
    def _ephy_downloads_folder_changed(self):
        dir = self.ephy_gconf_bridge.get("downloads_folder", "")
        if not dir:
            return

        self.ephy_directory_source = DirectorySource(dir)
        self.ephy_directory_source.connect("reload", lambda x: self.emit("reload"))
        self.emit("reload") # Notify listeners

    def get_enabled(self):
        return self.ephy_directory_source and self.ephy_directory_source.get_enabled()

    def get_items(self):
        if self.ephy_directory_source:
            temp_list=[]
            for item in self.ephy_directory_source.get_items():
                temp_list.append( item)
               # delitem
        gc.collect()
        return temp_list


class MozillaDownloadsSource(ItemSource):
    def __init__(self, name = "Mozilla Downloads", icon = None):
        ItemSource.__init__(self, name=name, icon=icon)
        self.download_rdf_monitors = {}

    def get_download_rdf_paths(self):
        '''
        Returns a list of paths to Mozilla downloads.rdf files.  It only returns
        files in the "default" profile, to avoid porn.  Maybe this should be
        revisited.
        '''
        return glob.glob(os.path.expanduser("~/.mozilla/*/*default*/downloads.rdf"))

    def get_enabled(self):
        if not self.download_rdf_monitors:
            self._monitor_download_rdf_paths()
        return len(self.download_rdf_monitors) > 0

    def _monitor_download_rdf_paths(self):
        for download_rdf in self.get_download_rdf_paths():
            if not self.download_rdf_monitors.has_key(download_rdf):
                # Emit reload on download.rdf change
                mon = FileMonitor(download_rdf)
                mon.connect("event", lambda m, uri, ev: self.emit("reload"))
                mon.open()
                self.download_rdf_monitors[download_rdf] = mon

    def get_items_uncached(self):
        if not self.download_rdf_monitors:
            self._monitor_download_rdf_paths()

        temp_list=[]
        for download_rdf in self.download_rdf_monitors:
            try:
                # Read previous download locations
                download_doc = parse(download_rdf)
                li_list = download_doc.getElementsByTagName("RDF:li")

                for li_elem in li_list:
                    uri = str(li_elem.getAttribute("RDF:resource"))
                    # Sometimes contains absolute paths instead of URLs
                    if os.path.isabs(uri):
                        uri = "file://" + uri

                    temp_list.append( FileItem(uri))
            except (IOError, ExpatError), err:
                #print " !!! Error parsing Mozilla download file '%s': %s" % (download_rdf, err)
                pass
        gc.collect()
        return temp_list

class ThunderbirdAttachmentsSource(MozillaDownloadsSource):
    '''
    A MozillaDownloadsSource subclass that reads Thunderbird downloads.rdf files.
    '''
    def __init__(self):
        MozillaDownloadsSource.__init__(self, name=_("Attachments"), icon="stock_attach")

    def get_download_rdf_paths(self):
        '''Returns a list of paths to Mozilla Thunderbird downloads.rdf files.'''
        return glob.glob(os.path.expanduser("~/.*thunderbird/*/downloads.rdf"))


class DownloadsSource(RecentAggregate):
    '''
    Aggregate ItemSource subclass that lists files from: ~/Downloads,
    Mozilla/Firefox downloads.rdf files, and the Epiphany download directory.
    '''
    def __init__(self, name = _("Downloads"), icon = "stock_save"):
        moz_source = MozillaDownloadsSource()
        ephy_source = EpiphanyDownloadsSource()
        if os.environ.has_key("DOWNLOAD_PATH"):
            path = os.environ["DOWNLOAD_PATH"]
        else:
            path = "~/Downloads"
        download_dir_source = DirectorySource(path)

        RecentAggregate.__init__(self,
                                 name=name,
                                 icon=icon,
                                 sources=[moz_source, ephy_source, download_dir_source])
	

class RecentlyUsedDocumentsSource(RecentlyUsedOfMimeType):
    ### FIXME: This is lame, we should generate this list somehow.
    DOCUMENT_MIMETYPES = [
        # Covers:
        #   vnd.corel-draw
        #   vnd.ms-powerpoint
        #   vnd.ms-excel
        #   vnd.oasis.opendocument.*
        #   vnd.stardivision.*
        #   vnd.sun.xml.*
        re.compile("application/vnd.*"),
        # Covers: x-applix-word, x-applix-spreadsheet, x-applix-presents
        re.compile("application/x-applix-*"),
        # Covers: x-kword, x-kspread, x-kpresenter, x-killustrator
        re.compile("application/x-k(word|spread|presenter|illustrator)"),
        "application/ms-powerpoint",
        "application/msword",
        "application/pdf",
        "application/postscript",
        "application/ps",
        "application/rtf",
        "application/x-abiword",
        "application/x-gnucash",
        "application/x-gnumeric",
        ]
    
    def __init__(self):
        RecentlyUsedOfMimeType.__init__(self,
                                        name=_("Documents"),
                                        icon="stock_new-presentation",
                                        mimetype_list=self.DOCUMENT_MIMETYPES)
    def get_items_uncached(self):
        temp_list=[]
        for item in RecentlyUsedOfMimeType.get_items_uncached(self):
            temp_item = FileItem(uri=item.get_uri(), timestamp=item.get_timestamp())
            temp_list.append( temp_item)
           # delitem,temp_item
        gc.collect()
        return temp_list
        
class RecentlyUsedOthersSource(RecentlyUsedOfMimeType):
    ### FIXME: This is lame, we should generate this list somehow.
    DOCUMENT_MIMETYPES = [
        # Covers:
        #   vnd.corel-draw
        #   vnd.ms-powerpoint
        #   vnd.ms-excel
        #   vnd.oasis.opendocument.*
        #   vnd.stardivision.*
        #   vnd.sun.xml.*
        re.compile("text/*"),
        "application/x-asp",
        "application/x-bittorrent",
        "application/x-blender",
        "application/x-cgi",
        "application/x-dia-diagram",
        "application/x-dvi",
        "application/x-glade",
        "application/x-iso-image",
        "application/x-jbuilder-project",
        "application/x-magicpoint",
        "application/x-mrproject",
        "application/x-php",
        ]
    
    def __init__(self):
        RecentlyUsedOfMimeType.__init__(self,
                                        name=_("Other"),
                                        icon="applications-other",
                                        mimetype_list=self.DOCUMENT_MIMETYPES)
    def get_items_uncached(self):
        temp_list=[]
        items = RecentlyUsedOfMimeType.get_items_uncached(self)
        for item in items:
             temp_item = FileItem(uri=item.get_uri(), timestamp=item.get_timestamp())
             temp_list.append( temp_item)
            # delitem,temp_item
        gc.collect()
        return temp_list
        
class RecentlyUsedImagesSource(RecentlyUsedOfMimeType):
    ### FIXME: This is lame, we should generate this list somehow.
    DOCUMENT_MIMETYPES = [
        # Covers:
        #   vnd.corel-draw
        re.compile("application/vnd.corel-draw"),
        # Covers: x-kword, x-kspread, x-kpresenter, x-killustrator
        re.compile("application/x-k(illustrator)"),
        re.compile("image/*"),
        ]
    
    def __init__(self):
        RecentlyUsedOfMimeType.__init__(self,
                                        name=_("Images"),
                                        icon="image",
                                        mimetype_list=self.DOCUMENT_MIMETYPES)
    def get_items_uncached(self):
        temp_list=[]
        items = RecentlyUsedOfMimeType.get_items_uncached(self)
        for item in items:
             temp_item = FileItem(uri=item.get_uri(), timestamp=item.get_timestamp())
             temp_list.append( temp_item)
            # delitem,temp_item
        gc.collect()
        return temp_list
        


class RecentlyUsedMusicSource(RecentlyUsedOfMimeType):
    ### FIXME: This is lame, we should generate this list somehow.
    MEDIA_MIMETYPES = [
        re.compile("audio/*"),
        "application/ogg"
        ]

    def __init__(self):
        RecentlyUsedOfMimeType.__init__(self,
                                        name=_("Music"),
                                        icon="gnome-mime-audio",
                                        mimetype_list=self.MEDIA_MIMETYPES)
    def get_items_uncached(self):
        temp_list = []
        for item in RecentlyUsedOfMimeType.get_items_uncached(self):
             temp_item = FileItem(uri=item.get_uri(), timestamp=item.get_timestamp())
             temp_list.append( temp_item)
            # delitem,temp_item
        gc.collect()
        return temp_list
        
        
class RecentlyUsedVideoSource(RecentlyUsedOfMimeType):
    ### FIXME: This is lame, we should generate this list somehow.
    MEDIA_MIMETYPES = [
        re.compile("video/*"),
        "application/ogg"
        ]

    def __init__(self):
        RecentlyUsedOfMimeType.__init__(self,
                                        name=_("Movies"),
                                        icon="gnome-mime-video",
                                        mimetype_list=self.MEDIA_MIMETYPES)
    def get_items_uncached(self):
        #print("videos uncached")
        temp_list = []
        for item in RecentlyUsedOfMimeType.get_items_uncached(self):
             temp_item = FileItem(uri=item.get_uri(), timestamp=item.get_timestamp())
             temp_list.append( temp_item)
            # delitem,temp_item
        gc.collect()
        return temp_list
        



#
# Topic Implementation
#

class DocumentsTopic(Topic):
    '''
    Lists recently opened/edited documents from ~/.recently-used, and some other
    categories of files such as movies and music.  Placeholders for recent email
    attachments, marked emails, and browser downloads.  In the future categories
    displayed should be pluggable.
    '''
    def __init__(self, prefs = None):
        Topic.__init__(self,
                       _("Library"),
                       uri="topic://Documents",
                       icon="stock_new-template",
                       prefs=prefs)

        self.source_list=[]
        self.do_reload()
        #from mayanna_running import RunningDocuments
        #self.set_running_source_factory(lambda: RunningDocuments())
    def do_reload(self):
        self.source_list = [
                       RecentlyUsedDocumentsSource(),
                       RecentlyUsedImagesSource(),
                       RecentlyUsedVideoSource(),
                       RecentlyUsedMusicSource(),
                       RecentlyUsedOthersSource(),
                       tomboy_source, # Tomboy
                       DownloadsSource(),
                       ]

        attachments = ThunderbirdAttachmentsSource()
        if attachments.get_enabled():
            self.source_list.append(attachments)
            
        self.source_list.insert(0, RecentAggregate(sources=[x for x in self.source_list if x]))   
        self.source_list.insert(1,"---")

        gc.collect()
    
    def get_hint_color(self):
        return gtk.gdk.color_parse("#D4C85C")

    def get_toolbar_items(self, tooltips):
        tools = []

        btn = NewDocumentMenuButton(tooltips)
        btn.set_is_important(True)
        tools.append(btn)

        btn = PlacesMenuButton(tooltips)
        btn.set_is_important(True)
        tools.append(btn)

        ### Uncomment to include a trashcan in the toolbar
        #btn = TrashButton(tooltips)
        #btn.set_is_important(True)
        #tools.append(btn)

        if not mayanna_is_panel_applet():
            tools.append(None) # Spacer
            btn = PrintersMenuButton(tooltips)
            btn.set_is_important(True)
            tools.append(btn)

        return tools
